unit hiMainForm;

interface

{$I share.inc}

uses Windows,Kol,Share,Win,Messages{,KOLMHToolTip};

type
 THIPosition = procedure of object;
 THIMainForm = class(THIWin)
   private
     IsMain,First:boolean;
     BitMap: PBitMap;
     CurForm:PControl;

     procedure SetCaption(const Value:string);
     procedure SetBorderStyle(Value:byte);
     procedure SetWindowsState(Value:byte);
     procedure SetTaskBar(Value:byte);
     procedure SetIcon(value:HICON);
     procedure SetPicture(Value:HBITMAP);
     procedure SetAlphaBlendValue(Value:byte);

     procedure Load;

     procedure _onMouseDown(Sender: PControl; var Mouse: TMouseEventData); override;
     function  _onMessage( var Msg: TMsg; var Rslt: Integer ): Boolean; override;
     procedure _onShow(Obj:PObj);
     function _OnClose( Sender: PObj; Accept: Boolean ):boolean;
     procedure _OnPaint( Sender: PControl; DC: HDC );
   public
     _prop_DragForm:boolean;
     _prop_AlphaBlendValue:byte;
     _prop_WindowsState:byte;
     _prop_SavePosition:procedure(State:boolean) of object;
     _prop_SavePosName:string;
     _prop_SavePosAll:byte;
     _prop_TransparentColor:TColor;
     _prop_ShowIcon:boolean;
     _prop_ClientSize:boolean;
     _prop_Position:THIPosition;

     _data_Close:THI_Event;
     _event_onClick:THI_Event;
     _event_onActivate:THI_Event;
     _event_onDeactivate:THI_Event;
     _event_onShow:THI_Event;
     _event_onCreate:THI_Event;
     _event_onMove:THI_Event;
     _event_onPaint:THI_Event;
     _event_onClose:THI_Event;

     constructor Create(Parent:PControl);
     destructor Destroy; override;
     procedure Init; override;
     procedure Start;

     procedure None(State:boolean);
     procedure Registry(State:boolean);
     procedure IniFile(State:boolean);

     procedure poNone;
     procedure poCenter;

     procedure _work_doCaption(var Data:TData; Index:word);
     procedure _work_doRestore(var Data:TData; Index:word);
     procedure _work_doMinimize(var Data:TData; Index:word);
     procedure _work_doAlphaBlendValue(var Data:TData; Index:word);
     procedure _work_doClose(var Data:TData; Index:word);
     procedure _work_doVisible(var Data:TData; Index:word);
     procedure _work_doPicture(var Data:TData; Index:word);
     procedure _work_doFlashWindow(var _Data:TData; Index:word);
     procedure _work_doIcon(var _Data:TData; Index:word);

     procedure _work_doBorderStyle(var _Data:TData; Index:word);
     procedure _work_doShowModal(var _Data:TData; Index:word);
     procedure _work_doPlaceInTaskBar(var _Data:TData; Index:word);

     property _prop_Caption:string write SetCaption;
     property _prop_Icon:HICON write SetIcon;
     property _prop_TaskBar:byte write SetTaskBar;
     property _prop_Picture:HBITMAP write SetPicture;
     property _prop_BorderStyle:byte write SetBorderStyle;
 end;

 procedure CreateCoolControl(Handle:cardinal; BitMap:PBitMap; tr:TColor = clWhite);

implementation

constructor THIMainForm.Create;
begin
   {$ifdef SUPER_PARENT}if Parent<>nil then Parent := Parent.ParentForm;{$endif}
   inherited Create(Parent);
   if not Assigned(Applet) then
    begin
     Applet := NewApplet('');
     IsMain := true;
    end
   else IsMain := false;
   if FParent = nil then Control := NewForm(Applet,'Form')
   else Control := NewForm(FParent,'Form');
   with Control{$ifndef F_P}^{$endif} do
    begin
       Visible:=false;
       Font.FontName := 'MS Sans Serif';
       Font.FontHeight := -11;

       onShow := _onShow;
       Border := 0;
    end;
   InitAdd(Load);
   BitMap := NewBitmap(0,0);
end;

destructor THIMainForm.Destroy;
begin
  Bitmap.Free;
  inherited;
end;

procedure THIMainForm._work_doFlashWindow;
begin
  FlashWindow(Applet.Handle,true);
end;

procedure THIMainForm._work_doIcon;
begin
   if _IsIcon(_data) then
    begin
      Control.Icon := ToIcon(_data).handle;
      if isMain then
       Applet.Icon := Control.Icon;
    end;
end;

procedure THIMainForm._work_doBorderStyle;
begin
  SetBorderStyle(ToInteger(_Data));
end;

function THIMainForm._onClose;
var i:integer;
    p:PControl;
begin
   Result := true;
   if Accept and(ToIntegerEvent(_data_Close)<>0) then exit;
   for i := 0 to Control.ChildCount-1 do
     with Control.Children[i]{$ifndef F_P}^{$endif} do
       if isForm then Perform(WM_CLOSE,0,1);
   if CurForm <> nil then
     PostMessage(CurForm.Handle,WM_ACTIVATE,WA_ACTIVE,0 );
   CurForm := nil;
   
   if isMain then
    begin
     Result := false;
     _hi_OnEvent(_event_onClose);
     EventOff;
    end
   else
    begin
     p := FParent;
     {$ifndef SUPER_PARENT}
     while not p.isForm do p := p.Parent;
     {$endif}
     if p.ModalForm <> Control then
       Control.Hide
     else Control.ModalResult := -1;
    end;
   _prop_SavePosition(true);
end;

function THIMainForm._onMessage;
begin
   Result := false;
   case Msg.message of
     WM_ACTIVATE:
       if Msg.WParam > 0 then _hi_OnEvent(_event_onActivate)
       else if Msg.WParam = 0 then _hi_OnEvent(_event_onDeActivate);
     WM_CLOSE: Result := _onClose(Control,(Msg.lParam=0));
     WM_SIZE:
       if isMain and Assigned(Applet) then Applet.Width := Control.Width;
     WM_MOVE:
       begin
         if isMain and Assigned(Applet) then
           Applet.SetPosition(Control.Left,Control.Top);
         if Assigned(Control.OnPaint) then _hi_OnEvent(_event_onMove);
       end;
     else Result := inherited _onMessage(Msg,Rslt);
   end;

end;

procedure THIMainForm.SetCaption;
begin
   Control.Caption := Value;
   if IsMain and Assigned(Applet) then
     Applet.Caption := Value;
end;

procedure THIMainForm.SetBorderStyle;
const
  NmMask = not(WS_CAPTION or WS_THICKFRAME or WS_MAXIMIZEBOX or WS_MINIMIZEBOX);
  NmSet:array[0..7] of dword =(0,
  WS_CAPTION or WS_MINIMIZEBOX,
  WS_CAPTION or WS_THICKFRAME or WS_MAXIMIZEBOX or WS_MINIMIZEBOX,
  WS_CAPTION,
  WS_CAPTION,
  WS_CAPTION or WS_THICKFRAME,
  0,
  WS_THICKFRAME
  );
  ExMask = not(WS_EX_DLGMODALFRAME or WS_EX_WINDOWEDGE or WS_EX_TOOLWINDOW);
  ExSet:array[0..7] of dword =(0,0,0,
  WS_EX_WINDOWEDGE,
  WS_EX_TOOLWINDOW,
  WS_EX_TOOLWINDOW,
  WS_EX_DLGMODALFRAME,
  0
  );
begin
  if Value>7 then exit;
  with Control{$ifndef F_P}^{$endif} do
   begin
    GetWindowHandle;
    Style   := NmSet[Value]or(NmMask and Style);
    ExStyle := ExSet[Value]or(ExMask and ExStyle);
   end;
end;

procedure THIMainForm.SetWindowsState;
begin
   case Value of
    0:;
    1: Control.WindowState := wsMinimized;
    2: Control.WindowState := wsMaximized;
   end;
end;

procedure THIMainForm.SetTaskBar;
begin
  if IsMain then
   if (Value = 1)and Assigned(Applet) then
     Applet.ExStyle :={ Applet.ExStyle or} WS_EX_DLGMODALFRAME or WS_EX_TOOLWINDOW;
end;

procedure THIMainForm.SetIcon;
begin
   if isMain and Assigned(Applet) then
    Applet.Icon := Value;
   if Value = 0 then Value := FParent.Icon;
   Control.Icon := Value;
end;

procedure THIMainForm._OnPaint;
begin
   if (not BitMap.Empty)and(BitMap.PixelFormat <> pf1bit) then
      BitMap.Draw(DC,0,0);
   _hi_OnEvent(_event_onPaint,integer(Control.Handle));
end;

procedure CreateCoolControl;
  var
      p: ^TColor;//PDWord;
      x, y : Integer;
      FromX: Integer;
      RGN: HRGN;

  procedure AddSubLine;
    var PixelRGN: HRGN;
  begin
    PixelRGN:=CreateRectRgn(FromX,y,x,y+1);
    CombineRgn(RGN,RGN,PixelRGN,rgn_or);
    DeleteObject(PixelRGN);
  end;
begin
  BitMap.PixelFormat := pf32bit;
  RGN := CreateRectRgn(0,0,0,0);
  for y:=0 to BitMap.Height-1 do begin
    FromX:=-1;
    p:=BitMap.ScanLine[y];
    for x:=0 to BitMap.Width-1 do begin
      if p^ <> tr then begin
        if FromX<0 then FromX:=x;
      end else begin
        if FromX>=0 then begin
          AddSubLine;
          FromX:=-1;
        end;
      end;
      Inc(p);
    end;
    if FromX>=0 then AddSubLine;
  end;
  SetWindowRgn(Handle,RGN,True);
end;

procedure THIMainForm.SetPicture;
begin
  BitMap.Handle := Value;
  CreateCoolControl(Control.GetWindowHandle,Bitmap,_prop_TransparentColor);
end;

procedure THIMainForm.SetAlphaBlendValue;
begin
   Control.AlphaBlend := Value;
end;

procedure THIMainForm._onMouseDown;
begin
   if _prop_DragForm then
    begin
     ReleaseCapture;
     Control.Perform(WM_SYSCOMMAND, $F012, 0);
    end;
   inherited;
end;

procedure THIMainForm._onShow;
begin                // _debug('ok');
  if not First then
   begin
    SetWindowsState(_prop_WindowsState);
    _prop_SavePosition(false);
    First := true;
   end;
  _hi_OnEvent(_event_onShow);

end;

procedure THIMainForm.Load;
begin
  //Control.Perform( WM_USER,33,0);
  _hi_OnEvent(_event_onCreate);
  Control.Visible := _prop_Visible;
end;

procedure THIMainForm.Init;
var vsb:boolean;
begin
  if _prop_SavePosName='' then
    _prop_SavePosition := None
  else if Pos('.ini\',_prop_SavePosName)=0 then
    _prop_SavePosition := Registry
  else _prop_SavePosition := IniFile;

  vsb:= _prop_Visible;
  _prop_Visible:= false;
  inherited;
  _prop_Visible:= vsb;
  with Control{$ifndef F_P}^{$endif} do
   begin
    OnPaint := _OnPaint;
    SetAlphaBlendValue(_prop_AlphaBlendValue);
    if _prop_ClientSize then
      SetClientSize(_prop_Width,_prop_Height);
   end;
  _prop_Position;
end;

procedure THIMainForm.Start;
begin
  if Assigned(Applet) then
    Applet.Visible := _prop_Visible;
  EventOn;
  InitDo;
end;

procedure THIMainForm.None;
begin
end;

procedure THIMainForm.Registry;
var
  pkey: HKey;
  DataType, size, Disposition, tmp: Integer;
begin
  if State then
   begin
    if RegCreateKeyEx(HKEY_CURRENT_USER,PChar('Software\' + _prop_SavePosName), 0, nil,
       REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, nil, pkey, @Disposition) = ERROR_SUCCESS
     then begin
      tmp:=Control.Left;  RegSetValueEx(pkey, PChar('Left'),  0, REG_DWORD, @tmp, SizeOf(Integer));
      tmp:=Control.Top;   RegSetValueEx(pkey, PChar('Top'),   0, REG_DWORD, @tmp, SizeOf(Integer));
 if _prop_SavePosAll = 0 then   
      tmp:=Control.Height;
 if _prop_SavePosAll = 0 then
      RegSetValueEx(pkey, PChar('Height'),0, REG_DWORD, @tmp, SizeOf(Integer));
 if _prop_SavePosAll = 0 then
      tmp:=Control.Width;
 if _prop_SavePosAll = 0 then       
      RegSetValueEx(pkey, PChar('Width'), 0, REG_DWORD, @tmp, SizeOf(Integer));
      RegCloseKey(pkey);
     end;
   end
  else
   begin
    if RegOpenKeyEx(HKEY_CURRENT_USER,PChar('Software\'+_prop_SavePosName),0,KEY_READ,pkey) = ERROR_SUCCESS
     then begin
      size:=SizeOf(Integer);
      DataType:=REG_DWORD;
      if RegQueryValueEx(pkey,'Left',nil,@DataType,PByte(@tmp),@size)=ERROR_SUCCESS
      then Control.Left:=tmp;
      if RegQueryValueEx(pkey,'Top',nil,@DataType,PByte(@tmp),@size)=ERROR_SUCCESS
      then Control.Top:=tmp;
      if RegQueryValueEx(pkey,'Height',nil,@DataType,PByte(@tmp),@size)=ERROR_SUCCESS
      then Control.Height:=tmp;
      if RegQueryValueEx(pkey,'Width',nil,@DataType,PByte(@tmp),@size)=ERROR_SUCCESS
      then Control.Width:=tmp;
      RegCloseKey(pkey);
    end;
  end;
end;

procedure THIMainForm.IniFile;
var Ini:PIniFile; p:integer;
begin
  p := Pos('.ini\',_prop_SavePosName);
  Ini := OpenIniFile(GetStartDir + Copy(_prop_SavePosName,1,p+3));
  Ini.Section := Copy(_prop_SavePosName,p+5,MAXINT);
  with Control{$ifndef F_P}^{$endif} do
   if State then
    begin
     Ini.Mode := ifmWrite;
     Ini.ValueInteger('Left',Left);
     Ini.ValueInteger('Top',Top);
 if _prop_SavePosAll = 0 then
     Ini.ValueInteger('Width',Width);
 if _prop_SavePosAll = 0 then
     Ini.ValueInteger('Height',Height);
    end
   else
    begin
     Ini.Mode := ifmRead;
     Left := Ini.ValueInteger('Left',Left);
     Top := Ini.ValueInteger('Top',Top);
     Width := Ini.ValueInteger('Width',Width);
     Height := Ini.ValueInteger('Height',Height);
    end;
  Ini.Free;
end;

procedure THIMainForm.poNone;
begin
end;

procedure THIMainForm.poCenter;
begin
  Control.Left := (ScreenWidth - Control.Width) div 2;
  Control.Top := (ScreenHeight - Control.Height) div 2;
end;

procedure THIMainForm._work_doCaption;
var Str:string;
begin
  Str := ToString(Data);
  SetWindowText( Control.Handle, @Str[ 1 ] );
end;

procedure THIMainForm._work_doRestore;
begin
  Applet.WindowState := wsNormal;
end;

procedure THIMainForm._work_doMinimize;
begin
  Applet.WindowState := wsMinimized;
end;

procedure THIMainForm._work_doAlphaBlendValue;
begin
  Control.AlphaBlend := ToInteger(Data);
end;

procedure THIMainForm._work_doClose;
begin
  _prop_SavePosition(true);
  Control.Perform(WM_CLOSE,0,1);
end;

procedure THIMainForm._work_doVisible;
var i:integer;
begin
  _prop_Visible := ReadBool(Data);
  if not _prop_Visible then begin
    Control.Hide;
    if CurForm <> nil then
      PostMessage(CurForm.Handle,WM_ACTIVATE,WA_ACTIVE,0 );
    CurForm := nil;
  end else begin
    CurForm := Applet.ActiveControl;
    Control.Show;
    for i := 0 to Control.ChildCount-1 do
      Control.Children[i].Perform(WM_SIZE,0,0);
    end
end;

procedure THIMainForm._work_doShowModal;
var p:PControl;
begin
  p := Control.Parent;
  {$ifndef SUPER_PARENT}
  while not p.isForm do p := p.Parent;
  {$endif}
  Control.ShowModalParented(p);
end;

procedure THIMainForm._work_doPicture;
var bmp:PBitmap;
begin
   bmp := ToBitmap(Data);
   if bmp <> nil then
    begin
     BitMap.Assign(bmp);
     CreateCoolControl(Control.GetWindowHandle,bmp,_prop_TransparentColor);
     Control.Invalidate;
    end;
end;

procedure THIMainForm._work_doPlaceInTaskBar(var _Data:TData; Index:word);
begin
  Control.ExStyle := Control.ExStyle or WS_EX_APPWINDOW; 
end;

end.
