unit hiTrayIcon;

interface

uses Kol,Share,Windows,ShellAPI,Messages,Debug;

type
  THITrayIcon = class(TDebug)
   private
    ParentForm:PControl;
    FTrayData:TNotifyIconData;
    FInst:boolean;
    OldMessage:TOnMessage;
    FHint:string;

    Procedure AddTrayIcon(_hide:boolean = true);
    procedure RemoveTrayIcon;
    function OnMessage( var Msg: TMsg; var Rslt: Integer ): Boolean;
    procedure SetHint(const Value:string);
   public
    _prop_FormHook:byte;
    _prop_Header:String;
    _prop_Text:String;
    _prop_Icon:byte;

    _data_Header:THI_Event;
    _data_Text:THI_Event;

    _event_onDblClick:THI_Event;
    _event_onClick:THI_Event;
    _event_onMouseDown:THI_Event;
    _event_onMouseMove:THI_Event;
    _event_onMouseUp:THI_Event;

    constructor Create(Parent:PControl);
    destructor Destroy; override;
    procedure _work_doShow(var _Data:TData; Index:word);
    procedure _work_doHide(var _Data:TData; Index:word);
    procedure _work_doHint(var _Data:TData; Index:word);
    procedure _work_doIcon(var _Data:TData; Index:word);
    procedure _work_doAddTrayIcon(var _Data:TData; Index:word);
    procedure _work_doHeaderIcon(var _Data:TData; Index:word);
    procedure _work_doHeader(var _Data:TData; Index:word);
    procedure _work_doBalloon(var _Data:TData; Index:word);
    property _prop_Hint:string write SetHint;
  end;

type
  NotifyIconData_50 = record
    cbSize: DWORD;
    Wnd: HWND;
    uID: UINT;
    uFlags: UINT;
    uCallbackMessage: UINT;
    hIcon: HICON;
    szTip: array[0..MAXCHAR] of AnsiChar;
    dwState: DWORD;
    dwStateMask: DWORD;
    szInfo: array[0..MAXBYTE] of AnsiChar;
    uTimeout: UINT; // union with uVersion: UINT;
    szInfoTitle: array[0..63] of AnsiChar;
    dwInfoFlags: DWORD;
  end{record};

const
  NIF_INFO      =        $00000010;

  NIIF_NONE     =        $00000000;
  NIIF_INFO     =        $00000001;
  NIIF_WARNING  =        $00000002;
  NIIF_ERROR    =        $00000003;

type
  TBalloonTimeout = 10..30{seconds};
  TBalloonIconType = (bitNone,    //  
                      bitInfo,    //   ()
                      bitWarning, //   ()
                      bitError);  //   ()

function DZBalloonTrayIcon(const Window: HWND; const IconID: Byte; const Timeout: TBalloonTimeout; const BalloonText, BalloonTitle: String; const BalloonIconType: TBalloonIconType): Boolean; 

implementation

const
  TrayID = 1;
  CM_TRAYICON = WM_USER + 1;

constructor THITrayIcon.Create;
begin
   inherited Create;
   ParentForm := Parent;
   OldMessage := ParentForm.OnMessage;
   ParentForm.OnMessage := OnMessage;
end;

destructor THITrayIcon.Destroy;
begin
    //RemoveTrayIcon;
    inherited;
end;

procedure THITrayIcon.SetHint;
begin
  FHint := Value;
  // if FInst then
  //  begin
  //    RemoveTrayIcon;
  //    AddTrayIcon;
  //  end;
end;

function THITrayIcon.OnMessage;
begin
  Result := false;
  Case Msg.message Of
    WM_SYSCOMMAND:
      Case Msg.wParam Of
        SC_RESTORE : RemoveTrayIcon;
        SC_MINIMIZE: begin AddTrayIcon; Result := true; end;
      End;
    WM_DESTROY:
     Begin
      RemoveTrayIcon;
      ParentForm.OnMessage := OldMessage; //???
     End;
    WM_CLOSE:
     if (_prop_FormHook = 0)and(Msg.lParam=0) then
      begin
        SendMessage(ParentForm.Handle,WM_SYSCOMMAND,SC_MINIMIZE,0);
        Rslt := 0;
        Result := true;
      end;
    WM_SETICON:
     begin
        FTrayData.hIcon := ParentForm.Icon;
        Shell_NotifyIcon(NIM_MODIFY,@FTrayData);
     end;  
    CM_TRAYICON:
      Case Msg.lParam Of
        WM_LBUTTONDBLCLK: _hi_OnEvent(_event_onDblClick,0);   //SendMessage(ParentForm.Handle,WM_SYSCOMMAND,SC_RESTORE,0);
        WM_RBUTTONDBLCLK: _hi_OnEvent(_event_onDblClick,1);
        WM_LBUTTONUP: 
          begin                            
            _hi_OnEvent(_event_onMouseUp,0);
            _hi_OnEvent(_event_onClick,0);
          end;
        WM_RBUTTONUP: 
          begin
            _hi_OnEvent(_event_onClick,1);
            _hi_OnEvent(_event_onMouseUp,1);
          end;
        WM_LBUTTONDOWN: _hi_OnEvent(_event_onMouseDown,0);
        WM_RBUTTONDOWN: _hi_OnEvent(_event_onMouseDown,1);
        WM_MOUSEMOVE: _hi_OnEvent(_event_onMouseMove);
      End;
  End;
  Result := Result or _hi_OnMessage(OldMessage,Msg,Rslt);
end;

procedure THITrayIcon._work_doShow;
begin
   RemoveTrayIcon;
end;

procedure THITrayIcon._work_doHide;
//var IconData:TNotifyIconData;
begin
   {
   IconData.cbSize := SIZEOF(IconData);
   IconData.Wnd := ParentForm.Handle;
   IconData.uFlags := NIF_INFO;

   StrLCopy(FTrayData.szTip,PChar(ToString(_Data)),SizeOf(FTrayData.szTip)-1);
   IconData.uTimeout := 15000; // in milliseconds
   Shell_NotifyIcon(NIM_MODIFY,IconData);
   }
   AddTrayIcon;
end;

procedure THITrayIcon._work_doAddTrayIcon;
begin
   AddTrayIcon(false);
end;


Procedure THITrayIcon.AddTrayIcon;
Begin
  if _hide then
    ShowWindow(ParentForm.Handle,SW_HIDE);
  if ParentForm.Parent <> nil then
    ShowWindow(ParentForm.Parent.Handle,SW_HIDE);
  FTrayData.cbSize := SizeOf(FTrayData);
  FTrayData.Wnd := ParentForm.Handle;
  FTrayData.uID := TrayID;
  FTrayData.uFlags := NIF_ICON Or NIF_MESSAGE Or NIF_TIP;
  FTrayData.uCallBackMessage := CM_TRAYICON;
  FTrayData.hIcon := ParentForm.Icon;
  StrLCopy(FTrayData.szTip,PChar(FHint),SizeOf(FTrayData.szTip)-1);
  Shell_NotifyIcon(NIM_ADD,@FTrayData);
  FInst := true;
End;

procedure THITrayIcon.RemoveTrayIcon;
begin
   if FInst  then
    begin
      Shell_NotifyIcon(NIM_DELETE,@FTrayData);
      if ParentForm.Parent <> nil then
        ShowWindow(ParentForm.Parent.Handle,SW_SHOWNORMAL);
      ShowWindow(ParentForm.Handle,SW_SHOWNORMAL);
      SetForegroundWindow( ParentForm.Handle );
      FInst := false;
    end;
end;

procedure THITrayIcon._work_doHint;
begin
  FHint := ToString(_data);
  StrLCopy(FTrayData.szTip,PChar(FHint),SizeOf(FTrayData.szTip)-1);
  Shell_NotifyIcon(NIM_MODIFY,@FTrayData);
end;

procedure THITrayIcon._work_doIcon;
begin
  if _IsIcon(_data) then
    FTrayData.hIcon := ToIcon(_data).handle
  else FTrayData.hIcon := ParentForm.Icon;
  Shell_NotifyIcon(NIM_MODIFY,@FTrayData);
end;

procedure THITrayIcon._work_doHeaderIcon;
begin
  _prop_Icon := ToInteger(_Data);
end;

procedure THITrayIcon._work_doHeader;
begin
  _prop_Header := ToString(_Data); 
end;

procedure THITrayIcon._work_doBalloon;
var
 ico:TBalloonIconType;
 H,T:String;
begin
  case _prop_Icon of
   0: ico := bitNone;
   1: ico := bitInfo;
   2: ico := bitWarning;
   3: ico := bitError;
  end;
  H := ReadString(_Data,_data_Header,_prop_Header);
  T := ReadString(_Data,_data_Text,_prop_Text);
  DZBalloonTrayIcon(ParentForm.handle, 1, 10, T, H, ico);
end;

function DZBalloonTrayIcon;
const
  aBalloonIconTypes : array[TBalloonIconType] of Byte = (NIIF_NONE, NIIF_INFO, NIIF_WARNING, NIIF_ERROR);
var
  NID_50 : NotifyIconData_50;
begin
  FillChar(NID_50, SizeOf(NotifyIconData_50), 0);
  with NID_50 do begin
    cbSize := SizeOf(NotifyIconData_50);
    Wnd := Window;
    uID := IconID;
    uFlags := NIF_INFO;
    StrPCopy(szInfo, BalloonText);
    uTimeout := Timeout * 1000;
    StrPCopy(szInfoTitle, BalloonTitle);
    dwInfoFlags := aBalloonIconTypes[BalloonIconType];
  end{with};
  Result := Shell_NotifyIcon(NIM_MODIFY, @NID_50);
end;

end.
